<?php
/**
 * A generic controller class. You should extend this base class if you want to
 * write your own controller. You get the advantage that you can use the pre and
 * post filters.
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007 - 2009 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class BaseController extends BaseFrameworkSystem implements Registerable {
	/**
	 * Generic filter chains
	 */
	private $filterChains = array();

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();

		// Initialize both filter chains
		$this->initFilterChain('pre');
		$this->initFilterChain('post');

		// Add this controller to the registry
		Registry::getRegistry()->addInstance('controller', $this);
	}

	/**
	 * Private method to initialize a given filter chain
	 *
	 * @param	$filterChain	Name of the filter chain
	 * @return	void
	 */
	private function initFilterChain ($filterChain) {
		$this->filterChains[$filterChain] = ObjectFactory::createObjectByConfiguredName('filter_chain_class');
	}

	/**
	 * Adds a filter to the pre filter chain
	 *
	 * @param	$filterInstance		An instance of a filter
	 * @return	void
	 */
	public function addPreFilter (Filterable $filterInstance) {
		// Add the pre filter
		$this->filterChains['pre']->addFilter($filterInstance);
	}

	/**
	 * Adds a filter to the post filter chain
	 *
	 * @param	$filterInstance		An instance of a filter
	 * @return	void
	 */
	public function addPostFilter (Filterable $filterInstance) {
		// Add the post filter
		$this->filterChains['post']->addFilter($filterInstance);
	}

	/**
	 * Executes all pre filters
	 *
	 * @param	$requestInstance	An instance of a request class
	 * @param	$responseInstance	An instance of a response class
	 * @return	void
	 */
	protected function executePreFilters (Requestable $requestInstance, Responseable $responseInstance) {
		// Execute all pre filters
		$this->filterChains['pre']->processFilters($requestInstance, $responseInstance);
	}

	/**
	 * Executes all post filters
	 *
	 * @param	$requestInstance	An instance of a request class
	 * @param	$responseInstance	An instance of a response class
	 * @return	void
	 */
	protected function executePostFilters (Requestable $requestInstance, Responseable $responseInstance) {
		// Execute all post filters
		$this->filterChains['post']->processFilters($requestInstance, $responseInstance);
	}
}

// [EOF]
?>
